@@ -1,7 +1,9 @@
+# -*- coding: iso-8859-1 -*-
+#
 # Code to handle the RoutePlanner RPL3 database format
 # (Designed to be easily parsable in Python and humanly grokable too)
 #
-# Copyright (C) 1996-2002 Chris Lawrence
+# Copyright (C) 1996-2003 Chris Lawrence
 # This file may be freely distributed under the terms of the RoutePlanner
 # license.  A copy should appear as 'LICENSE' in the archive that this
 # file was included in.
@@ -13,6 +15,8 @@
 # db.cities - The list of city instances
 # db.routes - The list of route instances
 
+from __future__ import division
+
 import rpcity, rproute, string, time, os, re, sys, rpunits
 from rpunits import UNITS_METRIC, UNITS_US, UNITS_IMPERIAL, UNITS
 
@@ -23,7 +27,7 @@ except ImportError:
 
 VERID = "##VERSION##"
 VERSION = "RoutePlanner ##VERSION##" 
-COPYRIGHT = '(C) 1994-2002 Chris Lawrence'
+COPYRIGHT = '(C) 1994-2003 Chris Lawrence'
 
 classifications = {
     'XX/Other (speed)' : 0,
@@ -86,7 +90,7 @@ def ConvertToTime(road, cunits, codetable):
             
         speed = rpunits.Distance(spcode, cunits) + \
                 rpunits.Distance(road.speed, road.distance.units)
-        return 60.0 * float(road.distance/speed)
+        return 60.0 * (float(road.distance) / float(speed))
 
 def Efficiency(distance, units, speedcode, mpg, defmpg):
     distance = float(distance.AsUnit(units))
@@ -97,7 +101,7 @@ def Efficiency(distance, units, speedcode, mpg, defmpg):
 
     if units == UNITS_METRIC:
         try:
-            return our_mpg * distance / 100.0
+            return our_mpg * distance / 100
         except ZeroDivisionError, text:
             print 'Zero distance on segment: %s' % text
             return 0
@@ -139,14 +143,14 @@ def Preferred(road, tunits, codetable, prefertable, defaultprefer, flagbonus,
     return dist * 101.0 / (pref+1.0)
 
 def IsSameRoute(route1, route2):
-    if route1 == route2: return 1
+    if route1 == route2: return True
     #if '/' in route1: return 0
-    if '&' in route2: return 0
-    if not '/' in route2: return 0
+    if '&' in route2: return False
+    if not '/' in route2: return False
 
     # Common case: dissimilar multiplex (i.e. I-70/US 40)
     routes = string.split(route2, '/')
-    if route1 in routes: return 1
+    if route1 in routes: return True
 
     # Similar multiplex
     match = re.match(r'([A-Za-z]+)([- ])(\d+)$', route1)
@@ -168,10 +172,10 @@ def IsSameRoute(route1, route2):
 
             #print routes[start:end+1], num, num in routes[start:end+1]
 
-            if num in routes[start:end+1]: return 1
+            if num in routes[start:end+1]: return True
 
     # Weird case
-    return 0
+    return False
 
 ##    # Ok, we're looking for 'BIT-n/n/BIT/n/n' or 'BIT a/BIT/b/c'
 ##    # or the like.
@@ -214,8 +218,10 @@ class RPDatabase:
 
         lines = string.split(fh.read(), os.linesep)
         stuff = lines.pop(0)
-        if stuff[0:4] != 'RPL3':
+        if not stuff.startswith('RPL3') and not stuff.startswith('RPL4'):
             raise invalidformat, filename+' is not a valid RoutePlanner file'
+        unicodefmt = stuff.startswith('RPL4')
+        
         self.comment = string.strip(stuff[4:])
         if not self.comment:
             self.comment = ''
@@ -234,10 +240,16 @@ class RPDatabase:
 
         for i in range(cities):
             data = lines.pop(0)
-            fields = string.split(data, '|')
+            if unicodefmt:
+                data = data.decode('UTF-8')
+            else:
+                data = data.decode('iso-8859-1')
+                
+            fields = data.split('|')
+            
             x = rpcity.City( fields )
-            name = str(x)
-            cityhash[ name ] = cityhash[ string.lower(name) ] = (x, i)
+            name = unicode(x)
+            cityhash[ name ] = cityhash[ name.lower() ] = (x, i)
             citylist[ i ] = x
             if not (i % 100):
                 if not quiet:
@@ -259,7 +271,12 @@ class RPDatabase:
 
         for i in range(routes):
             data = lines.pop(0)
-            fields = string.split(data, '|')
+            if unicodefmt:
+                data = data.decode('UTF-8')
+            else:
+                data = data.decode('iso-8859-1')
+
+            fields = data.split('|')
             routelist[ i ] = rproute.Route( fields, cityhash )
             if not (i % 100):
                 if not quiet:
@@ -304,9 +321,9 @@ class RPDatabase:
 
         if progressbar:
             progressbar.set_label('Sorting...')
-            progressbar.set_current(0)
             progressbar.set_max(len(self.cities)+len(self.routes)+1)
             progressbar.show()
+            progressbar.set_current(0)
         
         self.Sort()
         
@@ -317,40 +334,42 @@ class RPDatabase:
             fh = gzip.GzipFile(filename, 'wb')
         else:
             fh = open(filename, 'w', -1)
-        
-        fh.write('RPL3 %s\n' % comment)
 
-        fh.write( '%d\n' % len(self.cities) )
+        print >> fh, 'RPL4', comment
+
+        print >> fh, len(self.cities)
         if progressbar:
             progressbar.set_label('Writing cities...')
             progressbar.set_current(1)
 
         for city in self.cities:
-            fh.write( `city`+'\n' )
+            print >> fh, city.utf8()
 
-        fh.write( '%d\n' % len(self.routes) )
+        print >> fh, len(self.routes)
         if progressbar:
             progressbar.set_label('Writing cities...')
             progressbar.set_current(len(self.cities)+1)
 
         for route in self.routes:
-            fh.write( `route`+'\n' )
+            print >> fh, route.utf8()
 
-        fh.write( self.author + '\n' + self.author_email + '\n' )
-        fh.write( self.units + '\n' + `self.classifications` + '\n' +
-                  `self.speed` + '\n' + `self.mpg` + '\n' +
-                  `self.prefer` + '\n' + `self.defmpg` + '\n' +
-                  `self.defprf` + '\n' )
+        print >> fh, self.author
+        print >> fh, self.author_email
+        print >> fh, self.units
+        print >> fh, self.classifications
+        print >> fh, self.speed
+        print >> fh, self.mpg
+        print >> fh, self.prefer
+        print >> fh, self.defmpg
+        print >> fh, self.defprf
         fh.close()
 
     def Rehash(self):
-        i = 0
-        for city in self.cities:
-            self.cityhash[str(city)] = (city, i)
-            i = i+1
+        for (i, city) in enumerate(self.cities):
+            self.cityhash[unicode(city)] = (city, i)
 
     def CitiesMatching(self, pat):
-        return map(lambda x, y=self: y.cities[x], self.CitiesMatchIndex(pat))
+        return [self.cities[x] for x in self.CitiesMatchIndex(pat)]
 
     def CitiesMatchIndex(self, pat, visible=()):
         x = self.cityhash.get(pat)
@@ -510,24 +529,24 @@ class RPDatabase:
         weightings = {}
 
         try:
-            #from space import hosieries
             from pqueue import PQueue
             Q = PQueue()
         except:
             # Use pure Python implementation
-            from pq3 import PQueue
-            Q = PQueue()
+            from pq3 import PQ0
+            Q = PQ0()
             Q.insert = Q.addelt
             Q.pop = Q.popsmallest
-        
+
         reverse_path = {start : start}
         costs = {start : 0}
-        try:
-            #from space import hosieries
-            from kjbuckets import kjSet
-            seen = kjSet()
-        except:
-            seen = {} # Emulate a kjSet
+##         try:
+##             #from space import hosieries
+##             from kjbuckets import kjSet
+##             seen = kjSet()
+##         except:
+##             seen = {} # Emulate a kjSet
+        seen = {}
 
         for neighbor in start.roads:
             pair = start, neighbor[2]
@@ -541,7 +560,7 @@ class RPDatabase:
                 weightings[(c1,c0)] = dist, road, 1-neighbor[1]
 
             Q.insert(wt[0], pair)
-            seen[pair] = 1
+            seen[pair] = True
 
         while len(Q):
             (priority, (left, right)) = Q.pop()
@@ -564,7 +583,7 @@ class RPDatabase:
 
                 for neighbor in right.roads:
                     pair = (right, neighbor[2])
-                    if not seen.has_key(pair):
+                    if pair not in seen:
                         wt = weightings.get(pair)
                         if not wt:
                             c0, c1 = pair
@@ -576,18 +595,18 @@ class RPDatabase:
                             weightings[(c1,c0)] = dist, road, 1-neighbor[1]
                         newwt = wt[0]+priority
                         Q.insert(newwt, pair)
-                        seen[pair] = 1
+                        seen[pair] = True
                         # Also add the reverse
                         pair = (pair[1], pair[0])
                         Q.insert(newwt, pair)
-                        seen[pair] = 1
+                        seen[pair] = True
         
         raise ValueError, "start unreachable from end"
 
 # End of RPDatabase
 
 def NicerTime(min):
-    return '%2d:%02d' % (min / 60, min % 60)
+    return '%2d:%02d' % divmod(min, 60)
 
 def ProduceTrail(trail, db, units, mpg, defmpg, speed):
     displist = []
@@ -726,13 +745,13 @@ statecodes = { 'us': {
     "BC" : "British Columbia",
     "MB" : "Manitoba",
     "NB" : "New Brunswick",
-    "NF" : "Newfoundland",
+    "NF" : "Newfoundland and Labrador",
     "NS" : "Nova Scotia",
     "NT" : "Northwest Territories",
     "NU" : "Nunavut",
     "ON" : "Ontario",
     "PE" : "Prince Edward Island",
-    "PQ" : "Qubec",
+    "PQ" : "Qubec", # Older abbreviation for Quebec
     "QU" : "Qubec",
     "SA" : "Saskatchewan",
     "YT" : "Yukon",
