@@ -1,65 +1,100 @@
 # Code for the GUI interface to RoutePlanner
 #
-# Copyright (C) 1996-2001 Chris Lawrence
+# Copyright (C) 1996-2004 Chris Lawrence
 # This file may be freely distributed under the terms of the RoutePlanner
 # license.  A copy should appear as 'LICENSE' in the archive that this
 # file was included in.
+#
+# $Id: rpgnome.py,v 1.14 2004/06/20 09:36:49 lordsutch Exp $
 
-import gnome, rpdbase
+import pygtk
+pygtk.require('2.0')
 
-gnome.app_id = 'RoutePlanner'
-gnome.app_version = rpdbase.VERID
+import rpdbase
 
-import string, sys, os, glob, commands, time, libglade, GDK
+import sys, os, glob, commands, time
 from rpcitylist import CityList
 from rpprogress import ProgressWin
 
-from gtk import *
-from gnome.ui import *
-from GDK import *
+import gtk
+import gtk.gdk
+import gnome
+import gnome.ui
+
+APP = 'RoutePlanner'
+VERSION = rpdbase.VERID
+COPYRIGHT = "Copyright (C) 1996-2004 Chris Lawrence"
+gnome.init(APP, VERSION)
+
+import gtk.glade
+import gobject
+
 from rpunits import *
-import gnome.config
+
+import gconf
+#import gnome.config
+
+# i18n
+DIR = 'i18n'
+
+import locale, gettext
+locale.setlocale (locale.LC_ALL, '')
+gettext.bindtextdomain (APP, DIR)
+gettext.textdomain (APP)
+gettext.install (APP, DIR, unicode=1)
+
+# Whether or not we use the new file selector
+newfilesel = 'FileChooserDialog' in dir(gtk)
 
 PANES = ['shortroute', 'quickroute', 'prefroute', 'leastfuel']
-GUIFILE = '/usr/share/routeplanner/routeplanner.glade'
+GUIFILE = '/usr/share/routeplanner/routeplanner.glade2'
 
-CONFIG_UNITS   = 'routeplanner/Defaults/units'
-CONFIG_DEFMPG  = 'routeplanner/Defaults/default_efficiency'
-CONFIG_DEFPREF = 'routeplanner/Defaults/default_preference'
+CONFIG_UNITS   = '/apps/routeplanner/Defaults/units'
+CONFIG_DEFMPG  = '/apps/routeplanner/Defaults/default_efficiency'
+CONFIG_DEFPREF = '/apps/routeplanner/Defaults/default_preference'
 
-CONFIG_TOLL   = 'routeplanner/Defaults/toll_penalty'
-CONFIG_FERRY  = 'routeplanner/Defaults/ferry_penalty'
-CONFIG_SCENIC = 'routeplanner/Defaults/scenic_bonus'
+CONFIG_TOLL   = '/apps/routeplanner/Defaults/toll_penalty'
+CONFIG_FERRY  = '/apps/routeplanner/Defaults/ferry_penalty'
+CONFIG_SCENIC = '/apps/routeplanner/Defaults/scenic_bonus'
 
 class Application:
     def __init__(self, filename=''):
-        if os.path.exists('routeplanner.glade'):
-            self.wtree = libglade.GladeXML('routeplanner.glade')
+        if os.path.exists('routeplanner.glade2'):
+            self.wtree = gtk.glade.XML('routeplanner.glade2')
         else:
-            self.wtree = libglade.GladeXML(GUIFILE)
+            self.wtree = gtk.glade.XML(GUIFILE)
         
         self.appwin = self.wtree.get_widget('app1')
-        self.appwin.connect("destroy", mainquit) 
-##        print "Instance self.wtree",self.wtree
-##        print "\n***Dictionary ot the instance were I can see self.wtree"
-##        print(self.__dict__)
-##        print "\nDictionary of the class were I can see on_about_cb"
-##        print(Application.__dict__)
-        # You need to create this diccionary in order to get the 
-        # the bound methods that you can autoconnect with.
-        # And quoting James: 
-        # "I know it is not perfect, but it should do what you want."
+        self.appwin.connect("destroy", gtk.main_quit) 
+
         dict = {}
         for key in dir(self.__class__):
             dict[key] = getattr(self, key)
 
         self.wtree.signal_autoconnect(dict)
         self.db = None
-        self.citylist = self.wtree.get_widget('citylist')
+        self.cityview = self.wtree.get_widget('citylist')
+
+        self.citystore = gtk.ListStore(gobject.TYPE_PYOBJECT,
+                                       gobject.TYPE_STRING)
+        renderer = gtk.CellRendererText()
+        col = gtk.TreeViewColumn('City', renderer, text=1)
+        self.cityview.append_column(col)
+        self.cityview.set_reorderable(True)
+        self.cityview.set_model(self.citystore)
+
+        sel = self.cityview.get_selection()
+        sel.set_mode(gtk.SELECTION_SINGLE)
+        sel.connect("changed", self.on_citylist_select_row)
+        
         self.citywin = None
         self.make_widgets_sensitive()
-        # Hardcoded for 0.1
-        self.units = gnome.config.get_string(CONFIG_UNITS)
+
+        self.gconf_client = gconf.client_get_default()
+        self.gconf_client.add_dir('/apps/routeplanner',
+                                  gconf.CLIENT_PRELOAD_RECURSIVE)
+
+        self.units = self.gconf_client.get_string(CONFIG_UNITS)
         if not self.units or self.units not in VALID_UNITS:
             self.units = UNITS_US
         
@@ -68,38 +103,38 @@ class Application:
                 'imperial' : UNITS_IMPERIAL}
         for widget, units in info.items():
             if units == self.units:
-                self.wtree.get_widget(widget).set_active(TRUE)
+                self.wtree.get_widget(widget).set_active(True)
 
         if self.units == UNITS_METRIC:
             self.wtree.get_widget('effunits').set_text('l/100km')
 
-        self.defmpg = gnome.config.get_int(CONFIG_DEFMPG)
+        self.defmpg = self.gconf_client.get_int(CONFIG_DEFMPG)
         if not self.defmpg or self.defmpg <= 0 or self.defmpg > 100:
             self.defmpg = 20
         effwidget = self.wtree.get_widget('defefficiency')
         effwidget.set_value(self.defmpg)
 
-        self.defpref = gnome.config.get_int(CONFIG_DEFPREF)
+        self.defpref = self.gconf_client.get_int(CONFIG_DEFPREF)
         if not self.defpref or self.defpref < 0 or self.defpref > 100:
             self.defpref = 50
         prefwidget = self.wtree.get_widget('defpref')
         prefwidget.set_value(self.defpref)
 
-        self.tollpen = gnome.config.get_int(CONFIG_TOLL)
+        self.tollpen = self.gconf_client.get_int(CONFIG_TOLL)
         if not self.tollpen or self.tollpen < -100 or self.tollpen > 100:
             self.tollpen = 0
         self.wtree.get_widget('avoidtolls').set_active(self.tollpen)
         self.wtree.get_widget('tollpen').set_sensitive(self.tollpen)
         self.wtree.get_widget('tollpen').set_value(self.tollpen)
 
-        self.ferrypen = gnome.config.get_int(CONFIG_FERRY)
+        self.ferrypen = self.gconf_client.get_int(CONFIG_FERRY)
         if not self.ferrypen or self.ferrypen < -100 or self.ferrypen > 100:
             self.ferrypen = 0
         self.wtree.get_widget('avoidferries').set_active(self.ferrypen)
         self.wtree.get_widget('ferrypen').set_sensitive(self.ferrypen)
         self.wtree.get_widget('ferrypen').set_value(self.ferrypen)
 
-        self.scenicbonus = gnome.config.get_int(CONFIG_SCENIC)
+        self.scenicbonus = self.gconf_client.get_int(CONFIG_SCENIC)
         if not self.scenicbonus or self.scenicbonus < -100 or \
            self.scenicbonus > 100:
             self.scenicbonus = 0
@@ -111,7 +146,7 @@ class Application:
             self.open_database(filename)
 
         prefswin = self.wtree.get_widget('prefswin')
-        prefswin.hide_on_delete(TRUE)
+        prefswin.hide_on_delete()
 
         classes = rpdbase.classifications.keys()
         classes.sort()
@@ -135,15 +170,17 @@ class Application:
                 speed, mpg, pref = 50, self.defmpg, self.defpref
 
             if cl[0:2] == 'XX': continue
-            x = GtkLabel(cl)
+            x = gtk.Label(cl)
             x.show()
             classtable.attach(x, 0, 1, i, i+1)
             row = []
             for j in range(1, 4):
                 type = ['','speed','efficiency', 'preference'][j]
-                x = GtkSpinButton()
-                cl2 = string.replace(cl, '/', '_')
-                val = gnome.config.get_int('routeplanner/'+type+'/'+cl2)
+                x = gtk.SpinButton()
+                cl2 = cl.replace('/', '_')
+                cl2 = cl2.replace(' ', '_')
+                val = self.gconf_client.get_int('/apps/routeplanner/'+type+
+                                                '/'+cl2)
                 maxval = 100
                 if not val:
                     if j == 1:
@@ -170,23 +207,23 @@ class Application:
                     self.prefer[classid] = val
 
             self.prefwidgets[cl] = row
-            i = i+1
+            i += 1
 
-        #print classtable.children()
         classtable.show()
-        #prefswin.show()
 
     def on_exit1_activate(self,*args):
-        mainquit()
+        gtk.main_quit()
 
     def on_about1_activate(self,*args):
-        about = self.wtree.get_widget('aboutbox')
-        about.close_hides(TRUE)
+        about = gnome.ui.About(
+            APP, VERSION, COPYRIGHT,
+            _("Highway trip planner based on Jim Butterfield's RoadRoute."),
+            ['Chris Lawrence <lawrencc@debian.org>',
+             'Jim Butterfield'],
+            [], '')
+        #gtk.gdk.pixbuf_new_from_file(LOGO)
         about.show()
 
-    def on_about1_close(self,*args):
-        self.wtree.get_widget('aboutbox').hide()
-
     def on_properties1_activate(self, *args):
         if self.db:
             message = 'Current database: %s\n%s\n'\
@@ -197,110 +234,111 @@ class Application:
         else:
             message = 'No database loaded.'
             
-        dialog = GnomeOkDialog(message, self.appwin)
-        dialog.run_and_close()
+        dialog = gtk.MessageDialog(self.appwin, buttons=gtk.BUTTONS_OK,
+                                   message_format=message)
+        dialog.run()
+        dialog.destroy()
 
     def on_addcity_clicked(self, *args):
-        if not self.citywin: self.citywin = CityList(self.wtree)
+        if not self.citywin:
+            self.citywin = CityList(self.wtree)
         self.citywin.startup('', self.db, self.appendcity)
 
     def on_citylist_button_press_event(self, button, event, *args):
-        if event.type == GDK._2BUTTON_PRESS and event.button == 1 \
-           and self.citylist.selection:
+        if event.type == gtk.gdk._2BUTTON_PRESS and event.button == 1 \
+           and self.cityview.get_selection().get_selected():
             self.on_changecity_clicked()
 
     def on_changecity_clicked(self, *args):
         if not self.citywin:
             self.citywin = CityList(self.wtree)
-        pos = self.citylist.selection[0]
-        cityname = self.citylist.get_text(pos, 0)
+        sel = self.cityview.get_selection()
+        city, it = sel.get_selected()
+        pos = city.get_path(it)[0]
+        cityname = city[pos][0]
         self.citywin.startup(cityname, self.db, self.changecity, pos)
 
     def appendcity(self, city, ind, *data):
-        self.citylist.append([city])
-        self.citylist.set_row_data(self.citylist.rows-1, ind)
-        self.citylist.select_row(self.citylist.rows-1, 0)
+        it = self.citystore.append()
+        self.citystore.set(it, 0, city, 1, unicode(city))
+        sel = self.cityview.get_selection()
+        sel.select_path(len(self.citystore)-1)
         self.make_widgets_sensitive()
 
     def changecity(self, city, ind, pos):
-        self.citylist.freeze()
-        self.citylist.insert(pos, [city])
-        self.citylist.set_row_data(pos, ind)
-        self.citylist.remove(pos+1)
-        self.citylist.select_row(pos, 0)
-        self.citylist.thaw()
-
-    def on_removecity_clicked(self, *args):
-        pos = self.citylist.selection[0]
-        self.citylist.remove(pos)
-        rows = self.citylist.rows
-        pos = min(rows-1, pos)
-        self.citylist.select_row(pos, 0)
+        sel = self.cityview.get_selection()
+        selcity, it = sel.get_selected()
+        pos = selcity.get_path(it)[0]
+        self.citystore[pos] = (city, unicode(city))
         self.make_widgets_sensitive()
 
-    def on_up_clicked(self, *args):
-        pos = self.citylist.selection[0]
-        self.citylist.swap_rows(pos, pos-1)
-        self.citylist.select_row(pos-1, 0)
-        self.update_ud_widgets()
-
-    def on_down_clicked(self, *args):
-        pos = self.citylist.selection[0]
-        self.citylist.swap_rows(pos, pos+1)
-        self.citylist.select_row(pos+1, 0)
-        self.update_ud_widgets()
-
-    def update_ud_widgets(self):
-        wlist = {'up': FALSE, 'down': FALSE,
-                 'changecity' : TRUE, 'removecity' : TRUE}
-        row = self.citylist.selection[0]
-        rows = self.citylist.rows
-        if row > 0:
-            wlist['up'] = TRUE
-        if row < rows-1:
-            wlist['down'] = TRUE
+    def on_removecity_clicked(self, *args):
+        sel = self.cityview.get_selection()
+        selcity, it = sel.get_selected()
+        if not it:
+            return
         
-        for widget, state in wlist.items():
-            self.wtree.get_widget(widget).set_sensitive(state)
+        pos = selcity.get_path(it)[0]
+        del self.citystore[pos]
+        self.make_widgets_sensitive()
 
-    def on_citylist_select_row(self, *args):
-        self.update_ud_widgets()
-
-    def on_citylist_unselect_row(self, *args):
-        wlist = ['up', 'down', 'changecity', 'removecity']
+    def on_citylist_select_row(self, sel, *args):
+        wlist = ['changecity', 'removecity']
+        if sel.get_selected()[1]:
+            state = True
+        else:
+            state = False
+        
         for widget in wlist:
-            self.wtree.get_widget(widget).set_sensitive(FALSE)
+            self.wtree.get_widget(widget).set_sensitive(state)
 
     def on_filesel_ok_clicked(self, *args):
         filename = self.fw.get_filename()
         self.fw.destroy()
         self.open_database(filename)
 
+    def on_filesel_cancel_clicked(self, *args):
+        self.fw.destroy()
+
     def on_open1_activate(self, *args):
-        self.fw = GtkFileSelection('Select database')
-        self.fw.set_transient_for(self.appwin)
-        self.fw.selection_entry.connect("activate",
-                                         self.on_filesel_ok_clicked)
-        self.fw.ok_button.connect("pressed", self.on_filesel_ok_clicked)
-        self.fw.cancel_button.connect("pressed", self.fw.destroy)
-        self.fw.set_modal(TRUE)
-        self.fw.hide_on_delete(TRUE)
-        self.fw.set_filename('/usr/share/routeplanner/')
-        self.fw.show()
+        if newfilesel:
+            fs = gtk.FileChooserDialog('Select database', self.appwin,
+                                       gtk.FILE_CHOOSER_ACTION_OPEN,
+                                       (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
+                                        gtk.STOCK_OPEN, gtk.RESPONSE_ACCEPT))
+            fs.set_current_folder('/usr/share/routeplanner')
+            response = fs.run()
+            fs.hide()
+            if response == gtk.RESPONSE_ACCEPT:
+                filename = fs.get_filename()
+                self.open_database(filename)
+            fs.destroy()
+        else:
+            self.fw = gtk.FileSelection('Select database')
+            self.fw.set_transient_for(self.appwin)
+            self.fw.selection_entry.connect("activate",
+                                             self.on_filesel_ok_clicked)
+            self.fw.ok_button.connect("pressed", self.on_filesel_ok_clicked)
+            self.fw.cancel_button.connect("pressed",
+                                          self.on_filesel_cancel_clicked)
+            self.fw.set_modal(True)
+            self.fw.hide_on_delete()
+            self.fw.set_filename('/usr/share/routeplanner/')
+            self.fw.show()
 
     def on_plan_clicked(self, *args):
         db = self.db
         path = []
-        for i in range(self.citylist.rows):
-            cname = self.citylist.get_text(i, 0)
-            cpos = self.citylist.get_row_data(i)
-            info = db.cities[cpos]
+        for (info, cname) in self.citystore:
             if path and info == path[-1]:
-                dialog = GnomeErrorDialog('Same city appears twice in a '
-                                          'row:\n'+cname+'\n'
-                                          'Please fix this and plan again.',
-                                          self.appwin)
-                dialog.run_and_close()
+                dialog = gtk.MessageDialog(self.appwin,
+                                           message_format='Same city appears '
+                                           'twice in a row:\n'+cname+'\n'
+                                           'Please fix this and plan again.',
+                                           type=gtk.MESSAGE_ERROR,
+                                           buttons=gtk.BUTTONS_CLOSE)
+                dialog.run()
+                dialog.destroy()
                 return
             path.append(info)
 
@@ -346,47 +384,46 @@ class Application:
         startloc.set_text(str(start))
         endloc.set_text(str(end))
         if via:
-            vialocs.set_text(string.join(map(str, via), '; '))
+            vialocs.set_text('; '.join(map(str, via)))
         else:
             vialocs.set_text('[none]')
 
-        font = load_font("fixed")
-        black = GdkColor(0,0,0)
+        #font = load_font("fixed")
+        #black = GdkColor(0,0,0)
 
         widgets = PANES
         for j in range(methods):
             route = self.wtree.get_widget(widgets[j])
-            conlen = route.get_length()
-            route.delete_text(0, conlen)
             text = rpdbase.FormatRoute(displist[j], path, self.units)
-            route.insert(font, black, None, text)
+            buff = gtk.TextBuffer()
+            route.set_buffer(buff)
+            tag = gtk.TextTag('routedesc')
+            tag.set_property('font', 'Monospace 10')
+            buff.get_tag_table().add(tag)
+            sob, eob = buff.get_bounds()
+            buff.insert_with_tags_by_name(eob, text, "routedesc")
             self.routes[j] = text
         
         planwin.connect("delete_event", self.close_route_win)
         planwin.show()
 
-    def on_copy1_activate(self, *args):
-        pane = self.wtree.get_widget('notebook1').get_current_page()
-        widget = PANES[pane]
-        self.wtree.get_widget(widget).copy_clipboard()
-
     def on_close1_activate(self, *args):
         self.close_route_win()
 
     def close_route_win(self, *args):
         self.wtree.get_widget('planwin').hide()
-        return TRUE
+        return True
 
     def on_print1_activate(self, *args):
         printwin = self.wtree.get_widget('printwin')
         printwin.connect("delete_event", self.close_print_win)
         printwin.set_transient_for(self.wtree.get_widget('planwin'))
-        printwin.hide_on_delete(TRUE)
+        printwin.hide_on_delete()
         printwin.show()
 
     def close_print_win(self, *args):
         self.wtree.get_widget('printwin').hide()
-        return TRUE
+        return True
 
     def on_print_ok_clicked(self, *args):
         self.wtree.get_widget('printwin').hide()
@@ -414,20 +451,36 @@ class Application:
         fh.close()
 
         self.wtree.get_widget('printwin').hide()
-        return TRUE
+        return True
 
     def on_save_as1_activate(self, *args):
-        self.fw = GtkFileSelection('Save routes as...')
-        self.fw.set_transient_for(self.wtree.get_widget('planwin'))
-        self.fw.selection_entry.connect("activate", self.save_as_done)
-        self.fw.ok_button.connect("pressed", self.save_as_done)
-        self.fw.cancel_button.connect("pressed", self.fw.destroy)
-        self.fw.set_modal(TRUE)
-        self.fw.hide_on_delete(TRUE)
-        self.fw.show()
+        if newfilesel:
+            fs = gtk.FileChooserDialog('Save routes as...',
+                                       self.wtree.get_widget('planwin'),
+                                       gtk.FILE_CHOOSER_ACTION_SAVE,
+                                       (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
+                                        gtk.STOCK_SAVE, gtk.RESPONSE_ACCEPT))
+            response = fs.run()
+            fs.hide()
+            if response == gtk.RESPONSE_ACCEPT:
+                filename = fs.get_filename()
+                do_save(filename)
+            fs.destroy()
+        else:
+            self.fw = gtk.FileSelection('Save routes as...')
+            self.fw.set_transient_for(self.wtree.get_widget('planwin'))
+            self.fw.selection_entry.connect("activate", self.save_as_done)
+            self.fw.ok_button.connect("pressed", self.save_as_done)
+            self.fw.cancel_button.connect("pressed", lambda x: self.fw.destroy())
+            self.fw.set_modal(True)
+            self.fw.hide_on_delete()
+            self.fw.show()
 
     def save_as_done(self, *args):
         fn = self.fw.get_filename()
+        self.do_save(fn)
+
+    def do_save(self, fn):
         if os.path.exists(fn):
             dialog = GnomeMessageBox("Overwrite existing file\n"+fn+"?",
                                      MESSAGE_BOX_QUESTION, STOCK_BUTTON_YES,
@@ -453,17 +506,17 @@ class Application:
 
     def make_widgets_sensitive(self):
         if not self.db:
-            wlist = {'addcity' : FALSE,
-                     'citylist' : FALSE,
-                     'plan' : FALSE}
-        elif self.citylist.rows > 1:
-            wlist = {'addcity' : TRUE,
-                     'citylist' : TRUE,
-                     'plan' : TRUE}
+            wlist = {'addcity' : False,
+                     'citylist' : False,
+                     'plan' : False}
+        elif len(self.citystore) > 1:
+            wlist = {'addcity' : True,
+                     'citylist' : True,
+                     'plan' : True}
         else:
-            wlist = {'addcity' : TRUE,
-                     'citylist' : TRUE,
-                     'plan' : FALSE}
+            wlist = {'addcity' : True,
+                     'citylist' : True,
+                     'plan' : False}
 
         for widget, state in wlist.items():
             self.wtree.get_widget(widget).set_sensitive(state)
@@ -482,13 +535,13 @@ class Application:
             return
 
         self.make_widgets_sensitive()
-        self.citylist.clear()
+        self.citystore.clear()
         self.appwin.set_title('RoutePlanner: '+filename)
         progwin.destroy()
 
     def on_preferences1_activate(self, *args):
         prefswin = self.wtree.get_widget('prefswin')
-        prefswin.close_hides(TRUE)
+        prefswin.close_hides(True)
         prefswin.show()
 
     def units_changed(self, *args):
@@ -526,7 +579,7 @@ class Application:
         self.currentunits = newunits
 
     def prefswin_changed(self, *args):
-        self.wtree.get_widget('prefswin').set_modified(TRUE)
+        self.wtree.get_widget('prefswin').set_modified(True)
 
     def on_prefswin_apply(self, *args):
         # Update prefs here
@@ -539,9 +592,9 @@ class Application:
         self.defmpg = self.wtree.get_widget('defefficiency').get_value_as_int()
         self.defpref = self.wtree.get_widget('defpref').get_value_as_int()
 
-        gnome.config.set_string(CONFIG_UNITS, self.units)
-        gnome.config.set_int(CONFIG_DEFMPG, self.defmpg)
-        gnome.config.set_int(CONFIG_DEFPREF, self.defpref)
+        self.gconf_client.set_string(CONFIG_UNITS, self.units)
+        self.gconf_client.set_int(CONFIG_DEFMPG, self.defmpg)
+        self.gconf_client.set_int(CONFIG_DEFPREF, self.defpref)
 
         self.tollpen = (self.wtree.get_widget('avoidtolls').get_active() and
                         self.wtree.get_widget('tollpen').get_value_as_int())
@@ -551,9 +604,9 @@ class Application:
         self.scenicbonus = (self.wtree.get_widget('usescenic').get_active()
                             and self.wtree.get_widget('scenicbonus').
                             get_value_as_int())
-        gnome.config.set_int(CONFIG_TOLL, self.tollpen)
-        gnome.config.set_int(CONFIG_FERRY, self.ferrypen)
-        gnome.config.set_int(CONFIG_SCENIC, self.scenicbonus)
+        self.gconf_client.set_int(CONFIG_TOLL, self.tollpen)
+        self.gconf_client.set_int(CONFIG_FERRY, self.ferrypen)
+        self.gconf_client.set_int(CONFIG_SCENIC, self.scenicbonus)
         
         items = self.prefwidgets.items()
         items.sort()
@@ -561,10 +614,12 @@ class Application:
             classid = rpdbase.classifications[cl]
             for j in range(1, 4):
                 type = ['','speed','efficiency', 'preference'][j]
-                val = row[j-1].get_value()
+                val = row[j-1].get_value_as_int()
 
-                cl = string.replace(cl, '/', '_')
-                gnome.config.set_int('routeplanner/'+type+'/'+cl, val)
+                cl = cl.replace('/', '_')
+                cl = cl.replace(' ', '_')
+                self.gconf_client.set_int('/apps/routeplanner/'+type+'/'+cl,
+                                          val)
 
                 if j == 1:
                     self.speeds[classid] = val
@@ -572,8 +627,6 @@ class Application:
                     self.mpgs[classid] = val
                 else:
                     self.prefer[classid] = val
-                
-        gnome.config.sync()
 
     def on_avoidtolls_toggled(self, widget, *args):
         self.wtree.get_widget('tollpen').set_sensitive(widget.get_active())
@@ -589,8 +642,10 @@ class Application:
 
     def on_prefswin_close(self, *args):
         self.wtree.get_widget('prefswin').hide()
-        return TRUE
+        return True
 
 def main(filename=''):
+    global app
     app = Application(filename)
-    mainloop()
+    gtk.main()
+    app.appwin.hide()
